package Padre::Plugin::Snippet::FBP::Preferences;

## no critic

# This module was generated by Padre::Plugin::FormBuilder::Perl.
# To change this module edit the original .fbp file and regenerate.
# DO NOT MODIFY THIS FILE BY HAND!

use 5.008005;
use utf8;
use strict;
use warnings;
use Padre::Wx             ();
use Padre::Wx::Role::Main ();

our $VERSION = '0.01';
our @ISA     = qw{
	Padre::Wx::Role::Main
	Wx::Dialog
};

sub new {
	my $class  = shift;
	my $parent = shift;

	my $self = $class->SUPER::new(
		$parent,
		-1,
		Wx::gettext("Snippet Preferences"),
		Wx::DefaultPosition,
		[ 245, 120 ],
		Wx::DEFAULT_DIALOG_STYLE,
	);

	$self->{snippets_checkbox} = Wx::CheckBox->new(
		$self,
		-1,
		Wx::gettext("Enable Snippets?"),
		Wx::DefaultPosition,
		Wx::DefaultSize,
	);

	$self->{static_line} = Wx::StaticLine->new(
		$self,
		-1,
		Wx::DefaultPosition,
		Wx::DefaultSize,
		Wx::LI_HORIZONTAL,
	);

	$self->{ok_button} = Wx::Button->new(
		$self,
		Wx::ID_OK,
		Wx::gettext("OK"),
		Wx::DefaultPosition,
		Wx::DefaultSize,
	);

	$self->{cancel_button} = Wx::Button->new(
		$self,
		Wx::ID_CANCEL,
		Wx::gettext("Cancel"),
		Wx::DefaultPosition,
		Wx::DefaultSize,
	);

	my $generated_code_sizer = Wx::BoxSizer->new(Wx::HORIZONTAL);

	my $button_sizer = Wx::BoxSizer->new(Wx::HORIZONTAL);
	$button_sizer->Add( 0, 0, 1, Wx::EXPAND, 5 );
	$button_sizer->Add( $self->{ok_button},     0, Wx::ALL, 5 );
	$button_sizer->Add( $self->{cancel_button}, 0, Wx::ALL, 5 );

	my $sizer = Wx::BoxSizer->new(Wx::VERTICAL);
	$sizer->Add( $generated_code_sizer,      0, Wx::EXPAND,           5 );
	$sizer->Add( $self->{snippets_checkbox}, 0, Wx::ALL,              5 );
	$sizer->Add( $self->{static_line},       0, Wx::EXPAND | Wx::ALL, 5 );
	$sizer->Add( $button_sizer,              0, Wx::EXPAND,           5 );

	$self->SetSizer($sizer);
	$self->Layout;

	return $self;
}

1;

# Copyright 2008-2012 The Padre development team as listed in Padre.pm.
# LICENSE
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl 5 itself.

